#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <windows.h>
#include "widgets.h"
#include "rawdisk.h"

static struct wgt_window *win;
static struct wgt_widget *lb_instto, *bn_inst, *bn_cancel, *cb_devs, *ck_usbonly;

static struct rawdisk_device rawdev[64];
static const char *items[64];
static int num_rawdev;


static void update_disks(void);
static void onclick(struct wgt_widget *w);
static void onmodify(struct wgt_widget *w);
static void ck_usbonly_handler(struct wgt_widget *w);


int WINAPI WinMain(HINSTANCE hinst, HINSTANCE hprevinst, char *cmdline, int showcmd)
{
	int x, y;
	MSG msg;


	if(!(win = wgt_window("256boss USB stick installer", 400, 300))) {
		return 1;
	}
	lb_instto = wgt_label(win, "Install to device:", 10, 10);

	x = wgt_xpos_after(lb_instto, WGT_AUTO);
	cb_devs = wgt_combo(win, items, num_rawdev, 0, x, 10, 200, WGT_AUTO, onmodify);

	y = wgt_ypos_after(cb_devs, 16);
	ck_usbonly = wgt_checkbox(win, "only show USB devices", 1, x, y,
			WGT_AUTO, WGT_AUTO, ck_usbonly_handler);

	y = wgt_ypos_after(ck_usbonly, 16);
	bn_inst = wgt_button(win, "Install", 10, y, WGT_AUTO, WGT_AUTO, onclick);

	x = wgt_xpos_after(bn_inst, WGT_AUTO);
	bn_cancel = wgt_button(win, "Cancel", x, y, WGT_AUTO, WGT_AUTO, onclick);

	update_disks();

	while(GetMessage(&msg, 0, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	wgt_destroy_window(win);
	return 0;
}

static int isusbdev(struct rawdisk_device *dev)
{
	char *ptr = dev->path;
	while(*ptr) {
		int i;
		char buf[3];
		for(i=0; i<3; i++) {
			buf[i] = tolower(ptr[i]);
		}
		if(memcmp(buf, "usb", 3) == 0) {
			return 1;
		}
		ptr++;
	}
	return 0;
}

static void update_disks(void)
{
	int i;

	for(i=0; i<num_rawdev; i++) {
		free(rawdev[i].name);
		free(rawdev[i].path);
	}

	if((num_rawdev = rawdisk_detect(rawdev, sizeof rawdev / sizeof *rawdev)) == -1) {
		MessageBox(0, "Failed to detect storage devices!", 0, MB_OK);
		exit(1);
	}

	if(wgt_checkbox_checked(ck_usbonly)) {
		for(i=0; i<num_rawdev; i++) {
			if(!isusbdev(rawdev + i)) {
				free(rawdev[i].name);
				free(rawdev[i].path);
				rawdev[i] = rawdev[--num_rawdev];
			}
		}
	}

	for(i=0; i<num_rawdev; i++) {
		items[i] = rawdev[i].name;
	}

	wgt_combo_setitems(cb_devs, items, num_rawdev);

	if(num_rawdev) {
		wgt_enable_widget(cb_devs);
		wgt_enable_widget(bn_inst);
	} else {
		wgt_disable_widget(cb_devs);
		wgt_disable_widget(bn_inst);
	}
}

static void onclick(struct wgt_widget *w)
{
	if(w == bn_inst) {
		MessageBox(0, "clicked", "clicked", MB_OK);
	} else if(w == bn_cancel) {
		PostQuitMessage(0);
	} else if(w == ck_usbonly) {
		update_disks();
	}
}

static void onmodify(struct wgt_widget *w)
{
	int sel = wgt_combo_selected(w);
	if(sel >= 0) {
		const char *selstr = wgt_get_combo_item(w, sel);
		MessageBox(0, selstr, "selected item", MB_OK);
	}
}

static void ck_usbonly_handler(struct wgt_widget *w)
{
	update_disks();
}
